{% extends 'base.html' %}
{% load pagination_tags %}

{% block title %}
    Blog Posts
{% endblock %}

{% block extra_head_meta %}
    <meta name="robots" content="noindex, follow">
{% endblock%}

{% block content %}

  <h1>Blog Posts</h1>

  <p><a href="{% url "info_blog_feed" %}">RSS Feed</a></p>

  {% for object in object_list %}
    <h2><a href="{{ object.get_absolute_url }}">{{ object.title }}</a></h2>

    <div>{{ object.publication_date|date }} ({{ object.publication_date|timesince }} ago)</div>

    {{ object.content_as_cleaned_html|safe }}


  {% endfor %}

  {% paginate %}

{% endblock %}
