{% extends "base.html" %}

{% block title %}
    {{ object.title }}
{% endblock %}

{% block content %}
    <h1 class="page-title">{{ object.title }}</h1>

    <div class="markdown infopage">
        {{ object.content_with_anchors|safe }}
    </div>
{% endblock %}

{% block extra_attribution %}
  {# The page contents may have come from Wikipedia. Display the attribution. #}
  {% include 'wikipedia_attribution.html' %}
{% endblock %}
