{% extends 'base.html' %}

{% block title %}
    {{ object.title }}
{% endblock %}

{% block open_graph %}{% include 'info/_blog_open_graph.html' %}{% endblock %}

{% block content %}


  <h1>{{ object.title }}</h1>

  <div>{{ object.publication_date|date }} ({{ object.publication_date|timesince }} ago)</div>

  {{ object.content_as_html|safe }}

  <p>(<a href="{% url "info_blog_feed" %}">RSS Feed</a>)</p>


{% endblock %}
